#include <pstream.h>
#include "initusb.h"

/*
 * 	initusb.cc (Initialize usb port)					mikeofthenight@yahoo.com
 *
 * 	A c++ program used in conjuction with a bash script to connect a broadband
 * 	usb dongle and then establish a connection to an ISP(internet service provider)
 * 	This program provides a timeout consisting of asterisks '*' at 1 second intervals
 * 	whilst waiting for the broadband dongle to be established with the usb port.
 *
 * 	Input: Initusb Vendor_ID:Product_ID
 *
 * 	Where both Vendor and Product are hexadecimal values separated by a full colon.
 * 	To obtain these values for your particular broadband dongle. First remove all
 * 	usb devices from your pc. Then insert your broadband dongle in a usb port. 
 * 	Wait at least 10 seconds then run from the command line: 
 * 	lsusb
 * 	It will display all of your usb ports similar to the following:
 * 	Bus 001 Device 019: ID 12d1:1003 Huawei Technologies Co., Ltd. E220 HSDPA Modem / E270 HSDPA/HSUPA Modem
 *  Bus 005 Device 001: ID 1d6b:0001 Linux Foundation 1.1 root hub
 *  Bus 004 Device 001: ID 1d6b:0001 Linux Foundation 1.1 root hub
 *  Bus 003 Device 001: ID 1d6b:0001 Linux Foundation 1.1 root hub
 *  Bus 002 Device 001: ID 1d6b:0001 Linux Foundation 1.1 root hub
 *
 *  The Vendor and Product IDs in this case are 12d1:1003
 *  Your values obtained must be placed in the bash script which calls this c++  program.
 *
 *  Returns:
 * 	Program returns the any one of the following:
 * 			0	usb connection established
 * 			1	no connection or waiting for connection
 * 			2 	an error of some sort ie: Timed out.
 *  The bash script obtains the return values by means of the $? option.
 *
 *  To compile this program first:
 *  Place the pstream.h file either in the include path to c++ headers
 *  example: /usr/include/c++/4.4.1/pstream.h. This path will differ for different versions of c++.
 *  Or place it in the current directory where the compiler can find it.
 *  The latest version of pstreams can be downloaded from sorceforge.net.
 *  
 *  Then run:   g++ -Wall -Os initusb.cc -o Initusb
 *              strip InitUsb
 *
 *  To test this program from the command line prior to using it in the bash script.
 *  run: ./Initusb 12d1:1003
 *  Replacing the vendor and product values to suit your broadband dongle.
 *  After the program times out, run at the command line prompt
 *  echo $?   It should return 2 Indicating a timeout condition.
 *  Next insert your broadband dongle and re-run the test.
 *  This time the program will exit before the timed out condition. If you then
 *  run at the command line prompt: echo $? it should return 0
 *
 *  Initusb should be placed along with Calert in an executable directory so that
 *  the bash script can find them.
 *  In my case its /home/mike/bin but they could be placed anywhere in the
 *  environment path.
 */

template class redi::basic_ipstream<char>;
template class redi::basic_pstreambuf<char>;

int main(int argc,char *argv[]) { 
using namespace edn;
int usb = 1;
int count = 0;
string UsbString;


SetupHandler(SIGINT, Si_handler);
SetupHandler(SIGQUIT,Sq_handler);

	if (argc != 2){
		cout << "\n\tusage: " << argv[0] << " Vendor:Product\n" 
		<< "\t\t(where vendor & product) are both in hexadecimal.\n"
		<< "\t\tExample: "<< argv[0] << " 12a4:e6c8\n";
		_exit(2);
		}
		UsbString += argv[1];
		cout << "\n\t\t\e[37;44m\033[1mBroadband  Dongle USB Connection  Script\033[0m\n";
		cout << "\n\t\t\tWaiting on usb port\n\e[16C"; // starting point for asterisks.

		for (count = 0; count < 40; delay(1),count++) {
			cout << "\e[0m\e[31m*\e[s\e[1A\n\e[u\e[0m";
			usb = usb_out(UsbString); 
		if (usb == 0) break;
		}
		if (count == 40) {
			cout << "\n\n\t\t\e[31mTimed out:\e[0m Usb port failed to establish.\n";
			cout << "\t\tRemove, re-insert the dongle and re-try.\n";
			_exit(2);
		}
		cout << "\n\t\t";

return usb;
}


int usb_out(std::string const& inString) {
using namespace edn;	
string ModemString;
string command="lsusb -d ";
string inHexdigit,str;
string::size_type idx;

		/* 
		 * Very rudimentary input error checking. Checks that the  command line argument contains
		 * only hex digits  with a full colon separator. The hexidecimal values returned by lsusb
		 * are transformed to lowercase for comparison.
		 */

			inHexdigit = inString;
			transform(inHexdigit.begin(),inHexdigit.end(),inHexdigit.begin(),tolower);
			idx = inHexdigit.find_first_of(":");

		if (idx == string::npos) {
			cerr << "\n\tHex digits must be separated with a full colon!\n" << endl;
			_exit(2);
			} 
			idx = inHexdigit.find_first_of(":",idx+1);

		if (idx != string::npos) {
			cerr << "\n\tOnly one full colon ':' separator is allowed!\n" << endl;
			_exit(2);
			} 
			idx = inHexdigit.find_first_not_of(HexString);

		if (idx != string::npos) {
			cerr << "\n\tOnly Hex digits [0-9 [a-f| A-F]] and a [:] character is allowed!\n" << endl;
			_exit(2);
			}	
	
			unsigned int status=0;
			command += inHexdigit;
			redi::ipstream ipp(command.c_str());
			ModemString.clear();
		while (ipp >> str) {
			ModemString.append(str);
			ModemString.append(" ");
			}
			ipp.close();
			if (ipp.rdbuf()->exited()){
			status = ipp.rdbuf()->status();
			}
		if (status > 0) return 1;
																            


		transform(ModemString.begin(),ModemString.end(),ModemString.begin(),tolower);
		idx = ModemString.find(inHexdigit);
	if (idx != string::npos) {
		return 0;  				/* usb modem established */
	} else {
		return 1;				/* usb modem failed to connect */
	}
}
